# Copyright 2016 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
"""Ops for fused Cudnn RNN models.

@@CudnnCompatibleGRUCell
@@CudnnCompatibleLSTMCell
@@CudnnGRU
@@CudnnLSTM
@@CudnnRNNRelu
@@CudnnRNNTanh
@@CudnnLSTMSaveable
@@CudnnGRUSaveable
@@CudnnRNNReluSaveable
@@CudnnRNNTanhSaveable
@@CudnnParamsFormatConverterLSTM
@@CudnnParamsFormatConverterGRU
@@CudnnParamsFormatConverterTanh
@@CudnnParamsFormatConverterRelu
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

# pylint: disable=unused-import,wildcard-import
from tensorflow.contrib.cudnn_rnn.python.layers import *
# pylint: enable=unused-import,wildcard-import


from tensorflow.python.util.all_util import remove_undocumented

_allowed_symbols = [
    "CudnnCompatibleGRUCell",
    "CudnnCompatibleLSTMCell",
    "CudnnGRU",
    "CudnnLSTM",
    "CudnnRNNRelu",
    "CudnnRNNTanh",
    "CudnnLSTMSaveable",
    "CudnnGRUSaveable",
    "CudnnRNNReluSaveable",
    "CudnnRNNTanhSaveable",
    "CudnnParamsFormatConverterLSTM",
    "CudnnParamsFormatConverterGRU",
    "CudnnParamsFormatConverterTanh",
    "CudnnParamsFormatConverterRelu",
]

remove_undocumented(__name__, _allowed_symbols)
